package zero1hd.rhythmbullet.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputMultiplexer;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.CoreMusicInfo;
import zero1hd.rhythmbullet.audio.map.GamePlayMap;
import zero1hd.rhythmbullet.stages.GameHUD;
import zero1hd.rhythmbullet.stages.GamePlayArea;

public class GameScreen extends ScreenAdapter {
	private GamePlayArea gameArea;
	private GameHUD gameHUD;
	
	private InputMultiplexer inputs;

	public RhythmBullet core;
	
	private CoreMusicInfo music;
	
	public GameScreen(RhythmBullet core, Screen screen) {
		this.core = core;
		
		// Overlay stuff
		ImageButton pause = new ImageButton(core.getDefaultSkin().getDrawable("pause"),
				core.getDefaultSkin().getDrawable("pause-down"));
		pause.setPosition(Gdx.graphics.getWidth() - pause.getWidth() - 25,
				Gdx.graphics.getHeight() - pause.getHeight() - 25);
		pause.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				pause();
			}
		});
		
		gameArea = new GamePlayArea(core.getAssetManager(), core.getPrefs());
		gameHUD = new GameHUD(core.getDefaultSkin(), 100f, gameArea, screen, core);
		inputs = new InputMultiplexer();
		inputs.addProcessor(gameHUD);
		inputs.addProcessor(gameArea);
		
	}
	
	public void setGamePlayMap(GamePlayMap gpm) {
		music = gpm.getMusicData();
		gameArea.setAudioMap(gpm);
		gameHUD.setMusic(gpm.getMusicData().getPlaybackMusic());
	}
	
	@Override
	public void show() {
		if (music == null) new NullPointerException("Idiot, you can't have a music game not have music on the gameplay screen...");
		Gdx.input.setInputProcessor(inputs);
		gameArea.loadShaders(core.getPrefs());
		
		if (!gameHUD.isPaused()) {
			music.getPlaybackMusic().play();
		}
		
		super.show();
	}

	@Override
	public void render(float delta) {
		Gdx.gl.glClearColor(0f, 0f, 0f, 0f);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
		//actual game and hud
		if (!gameHUD.isPaused()) {
			gameArea.act(delta);
			if (gameArea.getPolyjetEntity().isDead()) {
				end(false);
			}
			gameHUD.act(delta);
		}
		gameArea.getViewport().apply();
		gameArea.draw();
		gameHUD.getViewport().apply();
		gameHUD.draw();
		if (!music.getPlaybackMusic().isPlaying()) {
			end(true);
		}
		
		super.render(delta);
	}

	private void end(boolean trueEnd) {
		if (trueEnd) {
			// TODO they didn't die
		} else {
			// TODO They done goofed and ended up dying.
		}
	}

	@Override
	public void dispose() {
		gameHUD.dispose();
		gameArea.dispose();
		super.dispose();
	}
	
	@Override
	public void pause() {
		gameHUD.setPaused(true);
		super.pause();
	}
	
	@Override
	public void resize(int width, int height) {
		gameArea.getViewport().update(width, height, true);
		gameHUD.getViewport().update(width, height, true);

		super.resize(width, height);
	}
	
	public GamePlayArea getGameArea() {
		return gameArea;
	}
}